from .._core_exceptions import LograderError


class LograderStudentBuildError(LograderError):
    """
    This is the base exception class for all exceptions raised
    by the `lograder.dispatch` module, for easy error handling.
    """

    pass
